<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://www.weedesign.de
 * @since             1.0.0
 * @package           weebot
 *
 * @wordpress-plugin
 * Plugin Name:       weebot
 * Plugin URI:        http://www.weedesign.de/weebot.html
 * Description:       Bring AI to your website and generate your custom support chat bot
 * Version:           1.0.0
 * Author:            Wolfgang Ertl
 * Author URI:        http://wolfgang.ertl.weedesign.de
 * Text Domain:       weebot
 * Domain Path:       /languages
 */

 
/**
 * If this file is called directly, abort.
 */
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class.weebot.activator.php
 */
function activate_weebot() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class.weebot.database.php';
	weebot_Database::install();
	require_once plugin_dir_path( __FILE__ ) . 'includes/class.weebot.activator.php';
	weebot_Activator::activate();
}
register_activation_hook( __FILE__, 'activate_weebot' );

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class.weebot.deactivator.php
 */
function deactivate_weebot() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class.weebot.deactivator.php';
	weebot_Deactivator::deactivate();
}
register_deactivation_hook( __FILE__, 'deactivate_weebot' );

/**
 * The code that runs during plugin uninstall.
 * This action is documented in includes/class.weebot.database.php
 */
function uninstall_weebot($reinstall=false) {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class.weebot.database.php';
	weebot_Database::uninstall();
	if($reinstall===true) {
		weebot_Database::install();
	}
}
register_uninstall_hook( __FILE__, 'uninstall_weebot' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class.weebot.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_weebot() {

	$plugin = new weebot();
	$plugin->run();

}
run_weebot();
